import {GetAllDip, ViewDIP} from '@app/access/access.action';
import {DipModel, DlcmService} from '@app/generated-api';
import {CollectionTypedModel} from '@app/model/collection-typed.model';
import {Action, State, StateContext} from '@ngxs/store';
import {tap} from 'rxjs/internal/operators/tap';

export interface AccessStateModel {
  isLoading: boolean;
  totalDip: number;
  dips: DipModel[];
}

@State<AccessStateModel>({
  name: 'access',
  defaults: {
    totalDip: 0,
    isLoading: false,
    dips: [],
  },
})
export class AccessState {
  constructor(private dlcmService: DlcmService) {
  }

  @Action(GetAllDip)
  getAllDip(ctx: StateContext<AccessStateModel>, action: GetAllDip) {
    ctx.patchState({
      isLoading: true,
    });

    return this.dlcmService.accessDipGet().pipe(
      tap((collectionDips: CollectionTypedModel<DipModel>) => {
        ctx.patchState({
          dips: collectionDips._data,
          totalDip: collectionDips._page.totalItems,
          isLoading: false,
        });

        // Equals to :

        // const state = ctx.getState();
        // ctx.setState({
        //   ...state,
        //   dips: collectionDips.data as Dip[],
        //   isLoading: false,
        // });
      }),
    );
  }

  @Action(ViewDIP)
  viewDip(ctx: StateContext<AccessStateModel>, action: ViewDIP) {
    const state = ctx.getState();
    // ctx.patchState()
    ctx.setState({
      ...state,
      dips: [],
      isLoading: true,
    });
  }
}
