import {Component} from '@angular/core';
import {Router} from '@angular/router';
import {RoutesEnum} from '@app/module/app-routing.module';
import {AppService} from '@app/app.service';
import {TRANSLATE} from '@app/utils/translate.utils';
import {TranslateService} from '@ngx-translate/core';
import {OAuthService} from 'angular-oauth2-oidc';
import * as moment from 'moment';

@Component({
  selector: 'dlcm-root',
  templateUrl: './app.component.html',
  styleUrls: ['./app.component.scss'],
})
export class AppComponent {
  title = 'DLCM-Frontend';
  readonly defaultLanguage = 'en';

  routesEnum = RoutesEnum;

  constructor(private oauthService: OAuthService,
              private router: Router,
              private translate: TranslateService,
              private appService: AppService) {
    this.translateSetting();
    this.attemptToLogin();

    const messageBoxContent = TRANSLATE('messagebox.warning.text');
  }


  private attemptToLogin(): void {
    const option = {
      disableOAuth2StateCheck: true,
    };
    // tryLogin method just tries to parse the token(s) within the url when
    // the auth-server redirects the user back to the web-app
    // It doesn't send the user the the login page
    this.oauthService.tryLogin(option).then((isLoginAttemptSuccess) => {
      this.redirectToLogin(isLoginAttemptSuccess);
    });
  }

  private redirectToLogin(isLoginSuccess: boolean): void {
    const shouldRedirect = isLoginSuccess === false && this.oauthService.getAccessToken() === null;
    if (shouldRedirect) {
      this.router.navigate([RoutesEnum.login]);
    }
  }

  private translateSetting(): void {
    // this language will be used as a fallback when a translation isn't found in the current language
    this.translate.setDefaultLang(this.defaultLanguage);

    // the lang to use, if the lang isn't available, it will use the current loader to get them
    this.useLanguage(this.defaultLanguage);
  }

  useLanguage(language: string): void {
    this.translate.use(language);
  }

  getTime() {
    return moment().format('MMMM Do YYYY, h:mm:ss a');
  }
}
