import {HTTP_INTERCEPTORS, HttpClient, HttpClientModule} from '@angular/common/http';
import {ErrorHandler, NgModule} from '@angular/core';
import {BrowserModule} from '@angular/platform-browser';
import {BrowserAnimationsModule} from '@angular/platform-browser/animations';
import {DlcmInterceptor} from '@app/dlcm.interceptor';
import {ApiModule} from '@app/generated-api/api.module';
import {BASE_PATH} from '@app/generated-api/variables';
import {ErrorsHandlerService} from '@app/service/errors-handler.service';
import {SharedModule} from '@app/shared/shared.module';
import {TranslateLoader, TranslateModule} from '@ngx-translate/core';
import {TranslateHttpLoader} from '@ngx-translate/http-loader';
import {NgxsReduxDevtoolsPluginModule} from '@ngxs/devtools-plugin';
import {NgxsLoggerPluginModule} from '@ngxs/logger-plugin';
import {NgxsModule} from '@ngxs/store';
import {AuthConfig, JwksValidationHandler, OAuthModule, OAuthModuleConfig, OAuthStorage, ValidationHandler} from 'angular-oauth2-oidc';
import {environment} from '../environments/environment';
import {AppComponent} from './app.component';
import {AppService} from './app.service';
import {authConfig, authModuleConfig} from './auth.config';
import {HomeComponent} from './components/home/home.component';
import {LoginComponent} from './components/login/login.component';
import {PageNotFoundComponent} from './components/page-not-found/page-not-found.component';
import {AppRoutingModule} from './module/app-routing.module';

@NgModule({
  declarations: [
    AppComponent,
    HomeComponent,
    PageNotFoundComponent,
    LoginComponent,
  ],
  imports: [
    BrowserModule,
    BrowserAnimationsModule,
    AppRoutingModule,
    NgxsModule.forRoot([], {
      developmentMode: !environment.production,
    }),
    NgxsLoggerPluginModule.forRoot({
      logger: console,
      collapsed: true,
      disabled: environment.production,
    }),
    NgxsReduxDevtoolsPluginModule.forRoot({
      disabled: environment.production,
    }),
    ApiModule,
    HttpClientModule,
    OAuthModule.forRoot(authModuleConfig),
    TranslateModule.forRoot({
      loader: {
        provide: TranslateLoader,
        useFactory: HttpLoaderFactory,
        deps: [HttpClient],
      },
    }),
    SharedModule,
  ],
  providers: [
    AppService,
    {
      provide: OAuthModuleConfig,
      useValue: authModuleConfig,
    },
    {
      provide: ValidationHandler,
      useClass: JwksValidationHandler,
    },
    {
      provide: OAuthStorage,
      useValue: sessionStorage,
      // useClass: DlcmStorage
    },
    {
      provide: AuthConfig,
      useValue: authConfig,
    },
    {
      provide: BASE_PATH,
      useValue: environment.API_BASE_PATH,
    },
    {
      provide: HTTP_INTERCEPTORS,
      useClass: DlcmInterceptor,
      multi: true,
    },
    {
      provide: ErrorHandler,
      useClass: ErrorsHandlerService,
    },
  ],
  bootstrap: [AppComponent],
})
export class AppModule {
}


// required for AOT compilation
export function HttpLoaderFactory(http: HttpClient) {
  return new TranslateHttpLoader(http);
}
