import {Action, State, StateContext} from '@ngxs/store';
import {DepositService} from '@app/deposit/services/deposit.service';
import {AddDIP, GetAll} from '@app/deposit/deposit.action';
import {tap} from 'rxjs/internal/operators/tap';

export interface DepositStateModel {
  feed: boolean;
  dip: string;
  dips: [];
  isLoading: boolean;
}

@State<DepositStateModel>({
  name: 'deposit',
  defaults: {
    feed: false,
    dip: 'test',
    dips: [],
    isLoading: false,
  },
})
export class DepositState {
  constructor(private depositService: DepositService) {
  }

  @Action(AddDIP)
  addDIP(ctx: StateContext<DepositStateModel>, action: AddDIP) {
    // const state = ctx.getState();
    // ctx.setState({
    //   ...state,
    //   feed: !state.feed,
    //   dip: action.name,
    //   isLoading: true,
    // });

    // equals to
    // ctx.setState((state) => ({...state, dip: action.name}));

    // equals to
    ctx.patchState({
      dip: action.name,
    });

    // return ctx.dispatch(new GetAll());
  }

  @Action(GetAll)
  getAll(ctx: StateContext<DepositStateModel>, action: GetAll) {
    ctx.patchState({
      isLoading: true,
    });

    return this.depositService.getAll().pipe(
      tap((deposits: any) => {
        console.log('test', deposits);
        const state = ctx.getState();
        // ctx.patchState()
        ctx.setState({
          ...state,
          dips: deposits._data,
          isLoading: false,
        });
      }),
    );
  }
}
